float4x4 matWorld;
float4x4 matView;
float4x4 matProj;
float4x4 matTangent;
float4x4 matWorldViewProj;

float4 vecFog;
float4 vecViewPos;
float4 vecTime;
float4 vecColor;
float4 vecViewDir;
float4 vecSunDir;
float4 vecSkill41; // Wind_X, Wind_Y, Ripple, Scale

texture entSkin1;
texture entSkin2;
texture entSkin3;
texture mtlSkin1;
texture mtlSkin2;
texture mtlSkin3;
texture mtlSkin4;



sampler sReflectTex = sampler_state
{
	Texture = (mtlSkin1);
	AddressU = Clamp;
	AddressV = Clamp;
	
	MagFilter = Linear;
	MinFilter = Point;
	MipFilter = None;
};

sampler sRefractTex = sampler_state
{
	Texture = (mtlSkin2);
	AddressU = Clamp;
	AddressV = Clamp;
	
	MagFilter = Linear;
	MinFilter = Point;
	MipFilter = None;
};

sampler sAlphaTex = sampler_state { Texture = (entSkin1); };
sampler sBumpTex = sampler_state { Texture = (entSkin2); };

sampler sWaveTex = sampler_state { Texture = (mtlSkin4); };


//////////////////////////////////////////////////////////////////////
//section: transform - calculate vertex position /////////////////////
float4 DoTransform(float4 Pos)
{
	return mul(Pos,matWorldViewProj);
}

//////////////////////////////////////////////////////////////////////
//section: fog ///////////////////////////////////////////////////////
float DoFog(float4 Pos)
{
	float3 P = mul(Pos,matWorld);
	return 1 - (distance(P,vecViewPos)-vecFog.x) * vecFog.z;
}

//////////////////////////////////////////////////////////////////////
//section: pos - transform vertex position to world space ////////////
float4 DoPos(float4 Pos)
{
	return (mul(Pos,matWorld));
}
float3 DoPos(float3 Pos)
{
	return (mul(Pos,matWorld));
}

//////////////////////////////////////////////////////////////////////
//section: normal - transform vertex normal //////////////////////////
float3 DoNormal(float3 inNormal)
{
	return normalize(mul(inNormal,matWorld));
}

struct out_water_env // Output to the pixelshader
{
	float4 Pos	: POSITION;
	float  Fog	: FOG;
	
	float4 Tex0	: TEXCOORD0;	//Bumpmap,Shoremap
//	float4 Tex1 : TEXCOORD1;	//Shore1,Shore2
	float3 Tex2	: TEXCOORD2;	//Reflection
	float3 Tex3	: TEXCOORD3;	//Refraction
	float4 ViewDir	: TEXCOORD4;	//Refraction
	float3 Normal	: TEXCOORD5;	//Refraction
//	float4 Color : COLOR0;
};

out_water_env vs_water_env
(
	float4 inPos		: POSITION,
	float3 inNormal	: NORMAL,
	float2 inTex0		: TEXCOORD0
)
{
	out_water_env Out;
	
	Out.Pos = DoTransform(inPos); // transform to screen coordinates
	Out.Fog = DoFog(inPos);
	
	// bump and reflection coordinates
	Out.Tex0.xy = (inTex0 + vecSkill41.xy * vecTime.w) * vecSkill41.w;
	Out.Tex0.zw = inTex0;
	
//	Out.Tex1.xy = (inTex0 + vecSkill41.xy * sin(vecTime.w*ShoreSpeed)*ShoreBewegung) * ShoreScale;//moving the shore texture	
//	Out.Tex1.zw = (inTex0 + vecSkill41.xy * cos(vecTime.w*ShoreSpeed)*ShoreBewegung) * ShoreScale;//moving the shore texture
	
	Out.Tex2 = Out.Pos.xyw;
	
	Out.ViewDir = normalize(vecViewPos-Out.Pos); // direction towards camera
	Out.Normal = DoNormal(inNormal);
	
	// color and transparency
//	Out.Color = vecColor;
//	float3 N = DoNormal(inNormal); // normal world orientation


//	Out.Color.a = 1-(0.5 + (1.0 - dot(vecToView,N)))*0.45;
	//Out.Color.a = 450/length(vecViewPos-Out.Pos);
	
	float4 tpos = mul(float4(inPos.x,0,inPos.z,1), mul(matView,matProj));
	Out.Tex3 = tpos.xyz/tpos.w;
	Out.Tex3.xy = 0.5 + 0.5*Out.Tex3.xy*float2(1,-1);
	Out.Tex3.z = 0.33/Out.Tex3.z;
	
	return Out;
}


float4 PS_p0(	float4 Tex0 : TEXCOORD0, //Bumpmap,Shoremap
				//	float4 Tex1 : TEXCOORD1, //Shore1,Shore2
					float3 Tex2	: TEXCOORD2, //Reflection
					float3 Tex3	: TEXCOORD3, //Refraction
					float3 ViewDir	: TEXCOORD4, //Refraction
					float3 Normal	: TEXCOORD5 //Refraction
//					float4 Color : COLOR0
				) : COLOR 
{
	//Read ShoreMapPixel
	float4 ShoreMapColor = tex2D(sAlphaTex,Tex0.zw);
	
	// die Bumpmap mit der waterCoord Koordinaten
	float4 Bump = tex2D(sBumpTex, Tex0.xy) * 2 - 1;
//	Bump += tex2D(sWaveTex, Tex0.zw) * 2 - 1;
	
	//
	float2 temp = Tex2.xy/Tex2.z;
	float2 CoordsRefl = 0.5*(1.0+temp)+Bump.xy*vecSkill41.z;
	float2 CoordsRefr = 0.5*(1.0+temp)+Tex3.z*Bump.xy;
	
	// Reflektierte Wasseroberflche
	float4 reflection = tex2D(sReflectTex, CoordsRefl);
	float4 refraction = tex2D(sRefractTex, CoordsRefr);
	
	float3 Normal2 = tex2D(sBumpTex,Tex0.xy).rbg*2.0-1.0;
	float3 reflectionVector = -reflect(vecSunDir, Normal2.xyz);
	float specular = dot(normalize(reflectionVector), normalize(ViewDir.xyz));
	specular = pow(specular, 80);

	float fresnel = saturate(dot(ViewDir,Normal)*1.5);
	float4 reflrefr = lerp(reflection,refraction,fresnel)+specular*2;
	

//reflrefr=reflrefr

	
	reflrefr.a = 1.0;
	
/*	
	float3 Normal = tex2D(smpBump,In.TexCoord1.xy).rbg*2.0-1.0;

float3 reflectionVector = -reflect(normalize(float3(-1.0,1.0,-1.0)), Normal);
float specular = dot(normalize(reflectionVector), normalize(In.EyeDir.xyz));
specular = pow(specular, 100);

float2 disortion = Normal.xz*In.EyeDir.y*0.5;*/
/*	if(ShoreMapColor.r > 0)
	{
		//Shoreline
		float4 ShoreColor;
		float4 ShoreAColor = tex2D(sShoreTex,Tex1.xy);
		float4 ShoreBColor = tex2D(sShoreTex,Tex1.zw);
		ShoreColor = ShoreAColor+ShoreBColor-1;
		
		reflrefr = lerp(reflrefr,ShoreColor*1.3,ShoreMapColor.r);
	}*/
	
//	reflrefr.a *= (1-ShoreMapColor.g);
	
	// Output
	return reflrefr;
}

//////////////////////////////////////////////////////////////////
technique water_env
{
	pass one
	{
		AlphaBlendEnable = True;
		ZEnable = True;
		ZWriteEnable=True;
		cullmode = none;
		
		VertexShader = compile vs_1_1 vs_water_env();
		PixelShader = compile ps_2_0 PS_p0();
	}
}

technique fallback { pass one { } }